% Demo script that uses torque shape-context for detecting an object
% category based on its shape.
% NOTE 1: the code assumes the model contours have been obtained via 
% 'modelcontours/script_extractModelEdges.m'
% NOTE 2: the code uses the Parallel computing toolbox to speed things up. 
% Make sure it is running to get faster results.  
% NOTE 3: the Pb edge detector needs to be installed at
% 'external/segbench/' for the edge detection code to run
%
% If you use this code for academic purposes, please cite:
% Ching L. Teo, C. Fermüller and Y. Aloimonos. 
% "A Gestaltist approach to contour-based object recognition: Combining bottom-up and top-down cues". 
% International Journal of Robotics Research (IJRR), Accepted (2014)
%
% Copyright (C) 2014 Ching L Teo, University of Maryland College Park, [cteo-at-cs.umd.edu]
%
%    You can redistribute and/or modify this software for non-commercial use
%    under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%    For commercial use, contact the author for licensing options. 
%
% Please email me if you find bugs, or have suggestions or questions!

clear all;
close all;

%% Setup and parameters
addpath(genpath(fullfile(pwd,'external'))); % all external libraries
para=getPara();
para.test.dir_suffix = 'Saw2'; % target shape name
objsize_vect = para.test.objsize_vect; % torque scales to search = expected object size 

%% Load and process model - 'msk_img_cell', 'objcat_contours'
% See 'modelcontours/script_extractModelEdges.m' for extracting our model contour data 
load(fullfile(para.test.model.dir, ['train_ObjClassContours_' para.test.dir_suffix '.mat']));
model_hd =  im2bw(imread(fullfile(para.test.model.dir, [para.test.dir_suffix '.jpg'])));
model_hd = ~model_hd; model_edges = bwperim(model_hd,4);
[model_pts,model_grads] = getPtSamplesFromContours(model_edges,para.sc.samp_freq);
% update objcat_dat
objcat_dat{1,1} = model_pts; objcat_dat{2,1} = model_grads;
objcat_dat{3,1} = model_hd; objcat_dat{4,1} = objcat_contours;

%% Read in image and compute pb edges (or your favorite edge detector)
img = im2double(imread(fullfile(pwd,'data', 'img_in.jpg')));
[pb, pb_theta] = pbCGTG(img); edges=pb; [rImg,cImg,~] = size(img);

%% Preprocess edge map to extract contours
[img_curve, labeled_img_curve]=preprocessEdges(edges, para);

%% Compute original torque
contour_frag_votemap = ones(rImg, cImg); para.torque.doModTorque = 0;
[torqueFixations_mat_cell_i, torqueFinal_mat_i, torqueFixations_level_cell_i, ...
   edgesTorque_i, EW_map_rw_i, patchListXY_i] = ... 
   computeWeightedTorqueFixations(img, img_curve, contour_frag_votemap, objsize_vect, [], para);

% compute supporting torque contours
[maxTorqueSegmentation_dat_i, minTorqueSegmentation_dat_i] = computeFixationEdges(img_curve, edgesTorque_i,EW_map_rw_i, torqueFinal_mat_i, torqueFixations_mat_cell_i,...
        torqueFixations_level_cell_i,  patchListXY_i, para);
   
%% Matching via torque shape-context
[edgevotes_Max, edgevotes_Min] = getRelevantContoursFromFixations(maxTorqueSegmentation_dat_i, minTorqueSegmentation_dat_i,...
                    img_curve, objcat_dat, labeled_img_curve, objsize_vect, para);
                
%% Convert matching votes to edge weights
weightedEM_final = getWeightedEdges(img_curve, edgevotes_Min, edgevotes_Max, para);

%% Compute modulated torque
para.torque.doModTorque = 1; 
[torqueFixations_mat_cell, torqueFinal_mat, torqueFixations_level_cell,...
    edgesTorque, EW_map_rw, patchListXY, torqueVolume] =...
    computeWeightedTorqueFixations(img, img_curve, weightedEM_final, objsize_vect, torqueFinal_mat_i, para);  

%% Display results
figure, imagesc(weightedEM_final); colorbar; title('Weighted Edges');
dispTorqueMaxMin(torqueFinal_mat, torqueFixations_mat_cell,img, para);
             
