function [curve_enc] = getEnclosingContours(img_pl, pt_c, dTheta, dRad, img_size, para) 
% function that takes in a log-polar transformed binary image, and finds the first
% transitions from zero --> 1 to get closest enclosing contours
%
% USAGE
%   [curve_enc] = getEnclosingContours(img_pl, pt_c, dTheta, dRad, img_size, para) 
%
% INPUTS
%   img_pl              - polar transform of image
%   pt_c                - pole: center point [x,y]
%   dTheta              - theta resolution
%   dRad                - radial resolution
%   img_size            - original image size [H,W]
%   para                - parameters (see getPara.m)
%
% OUTPUTS
%   curve_enc           - enclosing curves [Cx2]
%
%
% Copyright (C) 2014 Ching L Teo, University of Maryland College Park, [cteo-at-cs.umd.edu]
%
%    You can redistribute and/or modify this software for non-commercial use
%    under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%    For commercial use, contact the author for licensing options. 
%
% Please email me if you find bugs, or have suggestions or questions!


% get the rad and theta coordinates of the first transition from zero->one
[~, ind_c] = max(img_pl, [], 1); % rad coordinates

% % sanity check -- cc should be all ones
% if max(cc) ~= min(cc)
%     error('max of first transition not all ones');
% end

f_trans_map = [ind_c.*dRad; (1:size(ind_c,2)).*dTheta];
% convert back to cartesian -- ensuring that they are within image
% boundaries
f_trans_map_x = f_trans_map(1,:).*cos(f_trans_map(2,:))+pt_c(1);
f_trans_map_y = f_trans_map(1,:).*sin(f_trans_map(2,:))+pt_c(2);

% check boundaries if exceed image size
ind_bmax_x = round(f_trans_map_x) > img_size(2);
f_trans_map_x(ind_bmax_x) = img_size(2);
ind_bmax_y = round(f_trans_map_y) > img_size(1);
f_trans_map_y(ind_bmax_y) = img_size(1);

ind_bmin_x = round(f_trans_map_x) <= 0;
f_trans_map_x(ind_bmin_x) = 1;
ind_bmin_y = round(f_trans_map_y) <= 0;
f_trans_map_y(ind_bmin_y) = 1;


% close the curve
f_trans_map_y = [f_trans_map_y, f_trans_map_y(1)];
f_trans_map_x = [f_trans_map_x, f_trans_map_x(1)];

% create curve fragment [y x], and interpolate it to get back smooth curve
curve_frag = [f_trans_map_y', f_trans_map_x'];
curve_frag_s = smoothContour_edges(curve_frag, para, size(curve_frag,1)*20); % must have more points

% return [x y] curves 
curve_enc = fliplr(curve_frag_s);

end