function [pos_mask, neg_mask, neg_maskseg, curve_frag_pos, curve_frag_neg] =...
    getProtoSegments(torque_curr, pt_c, patchSize_c, para)
% Extracts the positive and negative proto-segments per fixation
% points from torque values
%
% USAGE
%   [pos_mask, neg_mask, neg_maskseg, curve_frag_pos, curve_frag_neg] = getProtoSegments(torque_curr, pt_c, patchSize_c, para)
%
% INPUTS
%   torque_curr             - torque value map [HxW]
%   pt_c                    - patch center [x,y]
%   patchSize_c             - patch size [1x2]
%   para                    - parameters (see getPara.m)
%
% OUTPUTS
%   pos_mask                - positive torque segments enclosing pt_c [HxW]
%   neg_mask                - negative torque segments not enclosing pt_c [HxW]
%   neg_maskseg             - segments of other patches [HxW]
%   curve_frag_pos          - contour points that encloses postive torque
%   curve_frag_neg          - contour points that encloses negative torque
%
% Copyright (C) 2014 Ching L Teo, University of Maryland College Park, [cteo-at-cs.umd.edu]
%
%    You can redistribute and/or modify this software for non-commercial use
%    under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%    For commercial use, contact the author for licensing options. 
%
% Please email me if you find bugs, or have suggestions or questions!

[rImg, cImg]=size(torque_curr);
% get center values
tval_center = torque_curr(pt_c(2),pt_c(1));
% get positive mask -- values within 0.5 of tval center are admitted
% get negative mask -- values that changes from sign of the center
ind_pos = torque_curr <= para.protoseg.pos_pct*tval_center;%0.5*tval_center;
ind_neg = torque_curr <= 0;
% convert to log-polar
[pos_pl, ~, ~] = polartrans(ind_pos, 1000, 1000, pt_c(1), pt_c(2), 'linear', 'full');
[neg_pl, dTheta, dRad] = polartrans(ind_neg, 1000, 1000, pt_c(1), pt_c(2), 'linear', 'full');
% get nearest closed contours that surround the fixation points
curve_frag_pos = getEnclosingContours(pos_pl, pt_c, dTheta, dRad, [rImg, cImg], para); % [x y]
curve_frag_neg = getEnclosingContours(neg_pl, pt_c, dTheta, dRad, [rImg, cImg], para);
% sanity check -- points that exceed bounds are set to bounds
curve_frag_pos(find(round(curve_frag_pos(:,2)) <= 0), 2 ) = 1;
curve_frag_neg(find(round(curve_frag_neg(:,2)) <= 0), 2 ) = 1;
curve_frag_pos(find(round(curve_frag_pos(:,2)) > rImg), 2 ) = rImg;
curve_frag_neg(find(round(curve_frag_neg(:,2)) > rImg), 2 ) = rImg;
curve_frag_pos(find(round(curve_frag_pos(:,1)) <= 0), 1 ) = 1;
curve_frag_neg(find(round(curve_frag_neg(:,1)) <= 0), 1 ) = 1;
curve_frag_pos(find(round(curve_frag_pos(:,1)) > cImg), 1 ) = cImg;
curve_frag_neg(find(round(curve_frag_neg(:,1)) > cImg), 1 ) = cImg;
% convert curves to binary masks -- put into a function
ind_bound_pos = sub2ind([rImg, cImg], min(rImg,round(curve_frag_pos(:,2))), min(cImg,round(curve_frag_pos(:,1))));
ind_bound_neg = sub2ind([rImg, cImg], min(rImg, round(curve_frag_neg(:,2))), min(cImg, round(curve_frag_neg(:,1))));
img_bound_pos = false(rImg, cImg);img_bound_neg = false(rImg, cImg);
img_bound_pos(ind_bound_pos) = true;img_bound_neg(ind_bound_neg) = true;
% bridge some gaps using morphological operations if the bounding curves
% are not closed
CC_pos = bwconncomp(img_bound_pos);CC_neg = bwconncomp(img_bound_neg);
if CC_pos.NumObjects > 1,img_bound_pos = makeClosedContours(img_bound_pos);end
if CC_neg.NumObjects > 1,img_bound_neg = makeClosedContours(img_bound_neg);end
% gets shape mask
pos_mask = imfill(img_bound_pos, 'holes');neg_mask = imfill(img_bound_neg, 'holes');
% clean-up isolated pixels
pos_mask = bwmorph(pos_mask, 'clean');neg_mask = bwmorph(neg_mask, 'clean');
% sanity check -- ensure only 1 region exists in each mask
CC_pos = bwconncomp(pos_mask);CC_neg = bwconncomp(neg_mask);
% take largest region
if CC_pos.NumObjects > 1
    cTmpC = false(rImg, cImg, CC_pos.NumObjects);cTmpV = zeros(CC_pos.NumObjects,1);
    for kk=1:CC_pos.NumObjects
        cTmp = false(size(pos_mask));cTmp(CC_pos.PixelIdxList{1,kk})=true;
        cTmpC(:,:,kk)=cTmp;cTmpV(kk,1) = sum(cTmp(:));
    end
    indS = max(cTmpV);pos_mask = cTmpC(:,:,indS);
end

if CC_neg.NumObjects > 1
    cTmpC = false(rImg, cImg, CC_neg.NumObjects);cTmpV = zeros(CC_neg.NumObjects,1);
    for kk=1:CC_neg.NumObjects
        cTmp = false(size(neg_mask));cTmp(CC_neg.PixelIdxList{1,kk})=true;
        cTmpC(:,:,kk)=cTmp;cTmpV(kk,1) = sum(cTmp(:));
    end
    [~,indS] = max(cTmpV);neg_mask = cTmpC(:,:,indS);
end


% determine min value of torque
val_m = min(torque_curr(:));
neg_maskseg = torque_curr <= val_m*para.protoseg.neg_pct;

% remove neg_maskseg within patchsize of pt_c
imbb = false(rImg, cImg);top_l = max(1, pt_c(2)-round(patchSize_c(2)/2));
bottom_l = min(rImg, pt_c(2)+round(patchSize_c(2)/2));left_l = max(1, pt_c(1)-round(patchSize_c(1)/2));
right_l = min(cImg, pt_c(1)+round(patchSize_c(1)/2));imbb(top_l:bottom_l, left_l:right_l) = true;
neg_ii = and(neg_maskseg, imbb);neg_maskseg = xor(neg_ii, neg_maskseg);

end