function [scaleMap,scaleValue,scaleStackMin,scaleStackMax] = findMaxMinTorqueScale( scaleStack, patchList )
% determines the torque scale at each maxima and minima
% for square patches only
%
% USAGE
%   [scaleMap,scaleValue,scaleStackMin,scaleStackMax] = findMaxMinTorqueScale( scaleStack, patchList )
%
% INPUTS
%   scaleStack                  - torque volume over S scales [HxWxS]
%   patchList                   - torque search scales [Sx1]
% 
% OUTPUTS
%   scaleMap                    - masked location of scale maxima/minima [HxW]
%   scaleValue                  - masked maxima/minima scale [HxW]
%   scaleStackMin               - minima scales [HxW]
%   scaleStackMax               - maxima scales [HxW]
%
% Copyright (C) 2012 Cornelia Fermüller, University of Maryland College Park, [fer-at-cfar.umd.edu]
%
%    You can redistribute and/or modify this software for non-commercial use
%    under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%    For commercial use, contact the author for licensing options. 
%
% Please email me if you find bugs, or have suggestions or questions!

    
    scaleStackMax = nanmax( scaleStack, [], 3 );
    scaleStackMin = nanmin( scaleStack, [], 3 );
    
    height = size(scaleStack,1);
    width  = size(scaleStack,2);
    scales = size(scaleStack,3);

    if exist( 'patchList', 'var' ) ~= 1 || isempty( patchList )
        patchList = 1:scales;
    end
    
    flgMax = scaleStack==repmat( scaleStackMax,[1,1,scales] );
    flgMin = scaleStack==repmat( scaleStackMin,[1,1,scales] );

    scaleIndex = repmat( reshape( patchList, [1,1,scales] ), [height,width,1] );    
    scaleMapMax = nanmax( flgMax.*scaleIndex, [], 3 );
    scaleMapMin = nanmax( flgMin.*scaleIndex, [], 3 );
    

    msk = scaleStackMax>=-scaleStackMin;

    scaleValue = scaleStackMin;
    scaleValue(msk) = scaleStackMax(msk);

    scaleMap = -scaleMapMin;
    scaleMap(msk) = scaleMapMax(msk);
    
    mskNaN = prod(double(flgMax),3)==1 | prod(double(flgMin),3)==1;
    
    scaleMap( mskNaN ) = NaN;

    mskNaN = ( scaleStackMax ~= 0 ) & ( scaleStackMax == -scaleStackMin );
    scaleValue( mskNaN ) = NaN;
    
end
