function [maxTorqueSegmentation_dat, minTorqueSegmentation_dat] = computeFixationEdges(img_curve, edgesTorque, EW_map_rw, torqueFinal_mat, torqueFixations_mat_cell,...
    torqueFixations_level_cell,  patchListXY, para)
% compute for each fixation point, the strongest edges contributing to it
%
% USAGE
%   [maxTorqueSegmentation_dat, minTorqueSegmentation_dat, patchSizeMax, patchSizeMin] =...
%            computeFixationEdges(img_curve, edgesTorque,EW_map_rw, torqueFinal_mat, torqueFixations_mat_cell,torqueFixations_level_cell,  patchListXY, torqueVolume, para)
% 
% INPUTS
%   img_curve                   - edge image [HxW]
%   edgesTorque                 - edge bins [HxWx0]
%   EW_map_rw                   - edge weights [HxW]
%   torqueFinal_mat             - torque map [HxW]
%   torqueFixations_mat_cell    - torque maxima/minima indices {1x2}
%   torqueFixations_level_cell  - torque maxima/minima scales {1x2}
%   patchListXY                 - torque search scales [Sx2]
%   para                        - parameters (see getPara.m)
% 
% OUTPUTS
%   maxTorqueSegmentation_dat   - top N maxima torque data {Nx1}
%   minTorqueSegmentation_dat   - top N minima torque data {Nx1}
%
% returns data from segmentation: [edges_contribution, postive_masks (inside), negative_mask (outside)]
%
% Copyright (C) 2014 Ching L Teo, University of Maryland College Park, [cteo-at-cs.umd.edu]
%
%    You can redistribute and/or modify this software for non-commercial use
%    under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%    For commercial use, contact the author for licensing options. 
%
% Please email me if you find bugs, or have suggestions or questions!

[rImg, cImg] = size(img_curve);

% get torque results - levels, and max min values
patchMinLevelMap = torqueFixations_level_cell{1}{1,1};
patchMaxLevelMap = torqueFixations_level_cell{1}{1,2};
torqueMaxValueMap = torqueFixations_mat_cell{1}{1,2};
torqueMinValueMap = torqueFixations_mat_cell{1}{1,1};

% get associated max min torque levels
patchMaxLevels = patchMaxLevelMap(torqueMaxValueMap(:,1));
patchMinLevels = patchMinLevelMap(torqueMinValueMap(:,1));

% get patchsize associated with the levels
patchSizeMax = patchListXY(patchMaxLevels,:);
patchSizeMin = patchListXY(patchMinLevels,:);

% convert to max and min indices
[iiMax, jjMax] = ind2sub([rImg/para.torque.resizeFactor cImg/para.torque.resizeFactor],... 
    torqueMaxValueMap(:,1));
[iiMin, jjMin] = ind2sub([rImg/para.torque.resizeFactor cImg/para.torque.resizeFactor],... 
    torqueMinValueMap(:,1));

% get edge contribution (for min and max torques)
edgesMaxCont_cell = computeEdgeContribution_XY(edgesTorque, EW_map_rw, [iiMax, jjMax],  patchSizeMax, para);
edgesMinCont_cell = computeEdgeContribution_XY(edgesTorque, EW_map_rw, [iiMin, jjMin],  patchSizeMin, para);

% get edges + torque results per fixations (for segmentation) -- uses
% torqueVolume
maxTorqueSegmentation_dat = getTorqueSegmentationData(iiMax, jjMax, torqueFinal_mat, patchSizeMax, edgesMaxCont_cell, 0, para);
minTorqueSegmentation_dat = getTorqueSegmentationData(iiMin, jjMin, torqueFinal_mat, patchSizeMin, edgesMinCont_cell, 1, para);


end
