function [torqueFixations_mat_cell, torqueFinal_mat, torqueFixations_level_cell, edgesC, EW_map_rw, patchListXY, torqueVolume] =...
    computeWeightedTorqueFixations(img, img_curve, contour_frag_votemap, objsize_vect, torqueFinal_mat_i, para)
% Computes weighted torque with rectangular patches, returning fixations
% points (maxima and minima)
%
% USAGE
%   [torqueFixations_mat_cell, torqueFinal_mat, torqueFixations_level_cell, torqueFixations_mat_cell_rn, edgesC, EW_map_rw, patchListXY, torqueVolume] =...
%           computeWeightedTorqueFixations(img, img_curve, contour_frag_votemap, objsize_vect, torqueFinal_mat_i, para)
%
% INPUTS
%   img                         - rgb image [HxWx3]
%   img_curve                   - edge image [HxW]
%   contour_frag_votemap        - weighted edges [HxW] for modulated torque (set
%   to ones[HxW] if not)
%   objsize_vect                - torque search scale
%   torqueFinal_mat_i           - Initial torque detections for modulated
%   torque (set to [] if not)
%   para                        - parameters (see getPara.m)
% 
% OUTPUTS
%   torqueFixations_mat_cell    - torque fixation indices {1x2} 
%   torqueFinal_mat             - torque value map [HxW]
%   torqueFixations_level_cell  - torque fixation scale {1x2}
%   edgesC                      - edges, split into O orientation [HxWxO]
%   EW_map_rw                   - weighted edges [HxW]
%   patchListXY                 - torque search scales [Sx2]
%   torqueVolume                - torque volume over scales [HxWxS]
%
% Copyright (C) 2014 Ching L Teo, University of Maryland College Park, [cteo-at-cs.umd.edu]
%
%    You can redistribute and/or modify this software for non-commercial use
%    under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%    For commercial use, contact the author for licensing options. 
%
% Please email me if you find bugs, or have suggestions or questions!

[rIm, cIm] = size(img_curve);
%% compute approximate torque scale
maxPatchSize = max(objsize_vect);
patchListXY = [3:para.torque.patchSkip:maxPatchSize; 3:para.torque.patchSkip:maxPatchSize]';
%% Compute torque fixation based on weighted edges, using default scale
edgesC = computeEdges(img, img_curve); % eight possible directions for integral-torque
EW_map_rw = contour_frag_votemap;   
%% Compute edge-reweighted torque
% compute integral reweighted torque
torqueVolume = computeIntegralWeightedTorque(edgesC, EW_map_rw, patchListXY, para);
% creates boundary mask
b_mask = false(rIm, cIm);
b_mask(end-(para.border):end,:)=true; b_mask(1:para.border+1,:)=true;
b_mask(:,end-(para.border):end)=true; b_mask(:,1:para.border+1)=true;
ind_b = find(b_mask);
% get the correct value and level map
patchList_s = patchListXY(:,1);
[scaleMap, valueMap, ~, ~] = findMaxMinTorqueScale( torqueVolume, patchList_s );
% cleanup scaleMap - set to zeros those that are within the boundary
scaleMap(ind_b) = 0;
if size(patchListXY,1) == 1 % only 1 level
    patchMinLevelMap = ones(rIm, cIm);
    patchMaxLevelMap = ones(rIm, cIm); 
else
    % convert scale map to max and min level map
    scaleMap_max_m = scaleMap > 0;
    patchMaxLevelMap = (scaleMap - min(patchListXY(:)))./para.torque.patchSkip + 1;
    patchMaxLevelMap = patchMaxLevelMap.*scaleMap_max_m;

    scaleMap_min_m = scaleMap < 0;
    patchMinLevelMap = (-scaleMap - min(patchListXY(:)))./para.torque.patchSkip + 1;
    patchMinLevelMap = patchMinLevelMap.*scaleMap_min_m;
end  
% using valueMap, compute fixation pts
minFlags = imextendedmin(valueMap, abs(nanmin(valueMap(:)))/para.torque.extremaWidth); 
maxFlags = imextendedmax(valueMap, abs(nanmax(valueMap(:)))/para.torque.extremaWidth);
% remove points too close to each other
maxFlags = imfill(maxFlags,'holes');minFlags = imfill(minFlags,'holes');
maxFlags = bwmorph(maxFlags, 'shrink', Inf);minFlags = bwmorph(minFlags, 'shrink', Inf);
indMin = find(minFlags);indMax = find(maxFlags);
valMinTorque = valueMap(indMin);valMaxTorque = valueMap(indMax);
minTorque = [indMin, valMinTorque];maxTorque = [indMax, valMaxTorque];
% sanity check -- ensure values in min and max torque are -ves, +ves
tt_min = minTorque(:,2) >= 0;
tt_max = maxTorque(:,2) <= 0;
minTorque_c = minTorque(~tt_min,:);
maxTorque_c = maxTorque(~tt_max,:);
if sum(tt_min) > 0, maxTorque_c = [maxTorque_c; minTorque(tt_min,:)];end
if sum(tt_max) > 0, minTorque_c = [minTorque_c; maxTorque(tt_max,:)];end
% remove those entries that are zero..
tz_min = minTorque_c(:,2) == 0;tz_max = maxTorque_c(:,2) == 0;
minTorque_c = minTorque_c(~tz_min,:);maxTorque_c = maxTorque_c(~tz_max,:);
% check if minTorque_c or maxTorque_c has locations within the boundary
tf_minB = ismember(minTorque_c(:,1), ind_b);tf_maxB = ismember(maxTorque_c(:,1), ind_b);
minTorque_c = minTorque_c(~tf_minB,:); maxTorque_c = maxTorque_c(~tf_maxB,:); % remove
% sort min/max vals
torqueMinValueMap = sortrows(minTorque_c, 2);torqueMaxValueMap = flipud(sortrows(maxTorque_c, 2));
%% return results
torqueFinal_mat = valueMap;
%[y x scaley scalex tval]
torqueFixations_mat_cell = cell(1,1);
torqueFixations_mat_cell{1}{1,1} = torqueMinValueMap;torqueFixations_mat_cell{1}{1,2} = torqueMaxValueMap;
torqueFixations_level_cell = cell(1,1);
torqueFixations_level_cell{1}{1,1} = patchMinLevelMap;torqueFixations_level_cell{1}{1,2} = patchMaxLevelMap;

end
