function [scaleStack] = findMomentScaleStack_XY(momentArms, sumEdges, sumMoments, patchListXY, para)
% Find 3D stack of moments over different patch sizes listed in patchList.
% Works for rectangular patches
% NOTE: Moments are wrt to patch centers, not image corner. 
%
% USAGE
%   [scaleStack] = findMomentScaleStack_XY(momentArms, sumEdges, sumMoments, patchListXY, para)
%
% INPUTS
%   momentArms                  - torque moments [HxWx8]
%   sumEdges                    - edge integral image [HxWx8]
%   sumMoments                  - moment integral image [HxWx8]
%   patchListXY                 - torque search scale (patch size) [Sx2]
%   para                        - parameters (see getPara.m)
% 
% OUTPUTS
%   scaleStack                  - torque volume over S scales [HxWxS]
%
% Copyright (C) 2014 Ching L Teo, University of Maryland College Park, [cteo-at-cs.umd.edu]
%
%    You can redistribute and/or modify this software for non-commercial use
%    under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%    For commercial use, contact the author for licensing options. 
%
% Please email me if you find bugs, or have suggestions or questions!

if nargin < 5
    bDoWeightedNorm = 0; % do not do weightedNorm by default
    weightedFPower = 2; weightedCCNPower = 2; nonweightedFPower = 2;
else
    bDoWeightedNorm = para.torque.doModTorque; weightedFPower = para.test.weighted.fPower;
    weightedCCNPower = para.test.weighted.CCNPower; nonweightedFPower = para.test.nonweighted.fPower;
end
nbScales = size(patchListXY,1);[nbRows, nbCols, ~] = size(sumEdges);
scaleStack = zeros(nbRows, nbCols, nbScales); 

parfor iScale = 1:nbScales
	patchSizeXY = patchListXY(iScale,:);
    edgePatchHisto = findAllPatchHistos_XY(sumEdges, patchSizeXY);
    momentPatchFromCorner = findAllPatchHistos_XY(sumMoments, patchSizeXY);   
	momentPatchHisto = momentPatchFromCorner - momentArms .* edgePatchHisto;	% arms from pixel to top left corner
    allDirMomentHisto = sum(momentPatchHisto, 3); % sum 8 moments
    if bDoWeightedNorm
        CC = sum(edgePatchHisto, 3); CC_N = (CC - min(CC(:)))./(max(CC(:)-min(CC(:))));           
        CC_N = CC_N.^(weightedCCNPower); % larger to add in more weights for better edges (cannot be too large -- if not weaker ones will be overweighted)
        fPower = weightedFPower; 
    else
        CC_N = ones(size(allDirMomentHisto));
        fPower = nonweightedFPower;  % a smaller value is for smoothing -- good for larger objects
    end
     normalizer = 2 * (patchSizeXY(1)^(fPower));       
     scaleStack(:,:,iScale) = (allDirMomentHisto.*CC_N)/normalizer; % normalized radial moment   

end

end



