function [torqueSegmentation_dat] = getTorqueSegmentationData(iiPatch, jjPatch, tFinal, patchSize, edgesCont_cell, flagInvert, para)
% function to get the segmentation data [edge_contribution, pos_mask,
% neg_mask] for each fixation point
%
% USAGE
%   [samples, gradients] = getPtSamplesFromContours(edgeImage, frequency_of_sampling)
%
% INPUTS
%   iiPatch             - y coordinates of patch center (fixation pt) [Fx1]
%   jjPatch             - x coordinates of patch center (fixation pt) [Fx1]
%   tFinal              - torque value map [HxW]
%   patchSize           - scales per fixation pt [Fx2]
%   edgesCont_cell      - supporting edges {Fx1}
%   flagInvert          - flag to indicate if we need to invert values (for
%   sorting)
%   para                - parameters (see getPara.m)
%
% OUTPUTS
%   torqueSegmentation_dat      - torque data
% 
% Copyright (C) 2014 Ching L Teo, University of Maryland College Park, [cteo-at-cs.umd.edu]
%
%    You can redistribute and/or modify this software for non-commercial use
%    under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%    For commercial use, contact the author for licensing options. 
%
% Please email me if you find bugs, or have suggestions or questions!



% get edges + torque results per fixations (for segmentation)
nFixations = size(iiPatch,1);
torqueSegmentation_dat = cell(min(nFixations,para.eval.topN), 1); % [edges_contribution, pos_mask, neg_mask]
parfor i=1:min(nFixations,para.eval.topN) % consider only topN fixations
    pt_c = [jjPatch(i), iiPatch(i)]; %[x, y]   
    % get the correct torquemap
    torqueFinal_mat = tFinal; patchSize_c = patchSize(i,:);
    if flagInvert, torqueFinal_mat = -torqueFinal_mat; end
    % get the positive_mask, negative_mask for the fixation points (using
    % log-polar transform
    [pos_mask, ~, neg_maskseg, ~, ~] = getProtoSegments(torqueFinal_mat, pt_c, patchSize_c, para);
    % get current edge contribution -- invert if necessary
    edge_cont_c = edgesCont_cell{i,1};
    if flagInvert, edge_cont_c = -edge_cont_c; end
    % select patch size for edge contribution
    [rIm, cIm] = size(torqueFinal_mat); patchSize_cH = round(0.5.*patchSize(i,:));
    rH = patchSize_cH(2); cH = patchSize_cH(1);
    edge_cont_map_s = edge_cont_c(max(1,pt_c(2)-rH):min(rIm,pt_c(2)+rH),...
        max(1,pt_c(1)-cH):min(cIm,pt_c(1)+cH)); edge_cont_map = zeros(size(edge_cont_c));
    edge_cont_map(max(1,pt_c(2)-rH):min(rIm,pt_c(2)+rH),...
        max(1,pt_c(1)-cH):min(cIm,pt_c(1)+cH)) = edge_cont_map_s;

    out_temp = cell(1,5); % [edge_cont, pos_mask, neg_mask, patchSize, patchCenter]
    out_temp{1,1} = edge_cont_map;
    out_temp{1,2} = pos_mask;
    out_temp{1,3} = neg_maskseg; % real neg mask for segmentation
    out_temp{1,4} = patchSize(i,:);
    out_temp{1,5} = pt_c; %[x,y]
    torqueSegmentation_dat{i,1} = out_temp;
end

end