function [SC, meanDistance] = computeTorqueShapeContext( Xpoints, Ypoints, XCenter, YCenter,...
    distanceBinCount, angleBinCount, closeDist, farDist, rangeLimit)
% Computes torque shape-context that adds in an 'inside' and 'outside'
% angular bins
%
% USAGE
%   [SC, meanDistance] = computeTorqueShapeContext( Xpoints, Ypoints, XCenter, YCenter,...
%                     distanceBinCount, angleBinCount, closeDist, farDist, rangeLimit) 
%
% INPUTS
%   Xpoints                 - x coordinates of edges [Px1]
%   Ypoints                 - y coordinates of edges [Px1]
%   XCenter                 - x coordinates of torque center
%   YCenter                 - y coordinates of torque center
%   distanceBinCount        - number of radial bins [B]
%   angleBinCount           - number of angular bins [A]
%   closeDist               - minimum radial distance threshold
%   farDist                 - maximum radial distance threshold
%   rangeLimit              - max angles (radians) for considering inside vectors 
%
% OUTPUTS
%   SC                      - torque shape-context [Px(BxA)]
%   meanDistance            - mean of edge points 
%
% Copyright (C) 2014 Ching L Teo, University of Maryland College Park, [cteo-at-cs.umd.edu]
%
%    You can redistribute and/or modify this software for non-commercial use
%    under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%    For commercial use, contact the author for licensing options. 
%
% Please email me if you find bugs, or have suggestions or questions!

sampleCount = size(Xpoints,1);
% Create the distance squared matrix (between pixels)
distances = sqrt(dist2([Xpoints Ypoints], [Xpoints Ypoints]));
% Normalize over the mean distance between points
meanDistance = mean(mean(distances)); distances = distances / meanDistance;
% Create the bins using a logarithmic scale between dInner and dOuter.
distanceBinEdges = logspace(log10(closeDist), log10(farDist), distanceBinCount);
% Quantize distances over the number of bins describing distances:
distanceBins = zeros(sampleCount,sampleCount);
for i=1:distanceBinCount, distanceBins = distanceBins + (distances < distanceBinEdges(i));end
% Point matching
angles = atan2(Ypoints*ones(1,sampleCount)-ones(sampleCount,1)*Ypoints',Xpoints*ones(1,sampleCount)-ones(sampleCount,1)*Xpoints')';
angles_center = atan2(YCenter - Ypoints, XCenter - Xpoints); % angles towards center pt
% Set angles to range from 0 to 2pi
angles = (angles >= 0) .* angles + ((angles + (2*pi)) .* (angles < 0)); angles_center = (angles_center >= 0) .* angles_center + ((angles_center + (2*pi)) .* (angles_center < 0));
% Quantize angles over the number of bins describing the angle:
angleBins = 1+floor(angles ./ ((2*pi)/(angleBinCount))); angleCenterBins = 1+floor(angles_center ./ ((2*pi)/(angleBinCount)));
inHistogram = (distanceBins > 0); % points that fall inside the histogram for each
% Assign each point in the histogram to its appropriate bin:
finalBins = (distanceBins .* angleBinCount - (inHistogram .* angleBinCount)) + (inHistogram .* angleBins);
finalHist = hist(finalBins,0:(angleBinCount*distanceBinCount));
% Expand angular vectors (soft-weighting)
rangeAngles = angleBinCount/2 - rangeLimit; % we do not go all the way to 180deg
if rangeAngles > 0
    angleCenterRange = repmat(angleCenterBins, [1, rangeAngles+1]);
    shiftVect = -(rangeAngles/2):rangeAngles/2; angleCenterRange = angleCenterRange + repmat(shiftVect, [sampleCount, 1]);
    % change negatives < 0 and those positives > angleBinCount to be within [1 angleBinCount]
    [r1,c1] = find(angleCenterRange < 0 | angleCenterRange > 12); ind1 = sub2ind([sampleCount, size(shiftVect,2)], r1, c1);
    angleCenterRange(ind1) = mod(angleCenterRange(ind1), angleBinCount);
    % set 0 to bin angleBinCount
    [r2,c2] = find(angleCenterRange == 0); ind2 = sub2ind([sampleCount, size(shiftVect,2)], r2, c2);
    angleCenterRange(ind2) = angleBinCount;
    % for vector towards center, we do not care about distance, so add distanceBins are admitted
    dBins = 1:distanceBinCount;
    for kk=1:size(angleCenterRange,2)
        anglesCenterCol = angleCenterRange(:,kk);
        anglesCenterColBins = repmat(anglesCenterCol, [1, distanceBinCount]).*repmat(dBins, [sampleCount, 1]);
        histAnglesCol = hist(anglesCenterColBins', 0:(angleBinCount*distanceBinCount));
        finalHist = finalHist + histAnglesCol; % add in angular bins
    end
else % do not expand range
    dBins = 1:distanceBinCount;
    anglesCenterBins = repmat(angleCenterBins, [1, distanceBinCount]).*repmat(dBins, [sampleCount, 1]);
    finalHistCenter = hist(anglesCenterBins', 0:(angleBinCount*distanceBinCount));
    finalHist = finalHist + finalHistCenter;     % add in histCenter to finalHist
end
% finalbins arrayed: (angles in closest ring) (angles in second closest ring) .. (angles in farthest ring)
SC = finalHist(2:size(finalHist,1),:)';

end