function [model_to_features, sc_cost, mean_feature_dist, mean_model_dist] =...
    matchTorqueShapeContext_FFT(model_pts, model_grads, model_center, feature_pts, feature_grads, feature_center, continuity_coeff, curvature_coeff,...
    edge_orient, gradient_orient, matching_method, rotAngle, para)
% matches pairs of torque shape-contexts via viterbi 
%
% USAGE
%   [model_to_features, sc_cost, mean_feature_dist, mean_model_dist] =...
%               matchTorqueShapeContext_FFT(model_pts, model_grads, model_center, feature_pts, feature_grads, feature_center, continuity_coeff, curvature_coeff,...
%                       edge_orient, gradient_orient, matching_method, rotAngle, para)
% 
% INPUTS
%   model_pts               - model edge points [Mx2]
%   model_grads             - model gradients [Mx2]
%   model_center            - model center [x,y]
%   feature_pts             - feature edge points [Ex2]
%   feature_grads           - feature gradients [Ex2]
%   feature_center          - feature center [x,y]
%   continuity_coeff        - weights for continuity strength btw points (see
%   viterbi_match.m)
%   curvature_coeff         - weights for curvature strength btw points
%   (see viterbi_match.m)
%   edge_orient             - consider edge orientation as weights? - (0) no, (1) yes
%   gradient_orient         - consider gradient orientation ? - (0) no, (1)
%   model only, (2) both test and model
%   matching_method         - histogram matching method - (0) chi^2
%   (currently one method only)
%   rotAngle                - estimated rotation
%   para                    - parameters (getPara.m)
%
% OUTPUTS
%   model_to_features       - correspondenes from model to test points [1xM]
%   sc_cost                 - shape context matching cost [MxE]
%   mean_feature_dist       - mean feature shape-context distance
%   mean_model_dist         - mean model shape-context distance
%
% Copyright (C) 2014 Ching L Teo, University of Maryland College Park, [cteo-at-cs.umd.edu]
%
%    You can redistribute and/or modify this software for non-commercial use
%    under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%    For commercial use, contact the author for licensing options. 
%
% Please email me if you find bugs, or have suggestions or questions!

eps_dum = Inf; dummy_pt = 0;

if ~isempty(rotAngle)
    % Rotate feature_pts if desired
    R_c = [cosd(rotAngle), -sind(rotAngle); sind(rotAngle), cosd(rotAngle)];
    Points_c = R_c*[feature_pts(:,1)'; feature_pts(:,2)']; feature_pts_t(:,1) = Points_c(1,:);
    feature_pts_t(:,2) = Points_c(2,:); Center_c = R_c*[feature_center(1); feature_center(2)];
    feature_center_t(1) = Center_c(1,1);feature_center_t(2) = Center_c(2,1);
    Gradients_c = R_c*[feature_grads(:,1)'; feature_grads(:,2)'];
    feature_grads(:,1) = Gradients_c(1,:); feature_grads(:,2) = Gradients_c(2,:);
    [feature_sc, mean_feature_dist] = computeTorqueShapeContext(feature_pts_t(:,1), feature_pts_t(:,2),...
        feature_center_t(1),feature_center_t(2), para.sc.n_dist, para.sc.n_theta, para.sc.mindist, para.sc.maxdist, para.sc.angleRange);
    [model_sc, mean_model_dist] = computeTorqueShapeContext(model_pts(:,1), model_pts(:,2),...
       model_center(1), model_center(2), para.sc.n_dist, para.sc.n_theta, para.sc.mindist, para.sc.maxdist, para.sc.angleRange);
else
    [feature_sc, mean_feature_dist] = computeTorqueShapeContext(feature_pts(:,1), feature_pts(:,2),...
        feature_center(1),feature_center(2), para.sc.n_dist, para.sc.n_theta, para.sc.mindist, para.sc.maxdist, para.sc.angleRange);
    [model_sc, mean_model_dist] = computeTorqueShapeContext(model_pts(:,1), model_pts(:,2),...
        model_center(1), model_center(2), para.sc.n_dist, para.sc.n_theta, para.sc.mindist, para.sc.maxdist, para.sc.angleRange);
end

if matching_method == 0
    sc_cost = hist_cost_2(model_sc, feature_sc);
else
    error('unexpected matching method');
end

%Make the cost prohibitive for matching points with different gradient
%directions:
%gradient
%categories:    1- +x, +y
%               2- +x, -y
%               3- -x, +y
%               4- -x, -y
if (edge_orient)
    if (gradient_orient == 2)
        model_grad_cat_1 = find(model_grads(:,1) > 0 & model_grads(:,1) > model_grads(:,2));
        model_grad_cat_2 = find(model_grads(:,1) > 0 & model_grads(:,1) < model_grads(:,2));
        model_grad_cat_3 = find(model_grads(:,1) <= 0 & model_grads(:,1) > model_grads(:,2));
        model_grad_cat_4 = find(model_grads(:,1) <= 0 & model_grads(:,1) < model_grads(:,2));
        
        feature_grad_cat_1 = find(feature_grads(:,1) > 0 & feature_grads(:,1) > feature_grads(:,2));
        feature_grad_cat_2 = find(feature_grads(:,1) > 0 & feature_grads(:,1) < feature_grads(:,2));
        feature_grad_cat_3 = find(feature_grads(:,1) <= 0 & feature_grads(:,1) > feature_grads(:,2));
        feature_grad_cat_4 = find(feature_grads(:,1) <= 0 & feature_grads(:,1) < feature_grads(:,2));
        
        sc_cost_with_eo = sc_cost;
        sc_cost_with_eo(model_grad_cat_1, [feature_grad_cat_2; feature_grad_cat_3; feature_grad_cat_4]) = eps_dum;
        sc_cost_with_eo(model_grad_cat_2, [feature_grad_cat_1; feature_grad_cat_3; feature_grad_cat_4]) = eps_dum;
        sc_cost_with_eo(model_grad_cat_3, [feature_grad_cat_1; feature_grad_cat_2; feature_grad_cat_4]) = eps_dum;
        sc_cost_with_eo(model_grad_cat_4, [feature_grad_cat_1; feature_grad_cat_2; feature_grad_cat_3]) = eps_dum;

        sc_cost_with_eo([model_grad_cat_2; model_grad_cat_3; model_grad_cat_4], feature_grad_cat_1) = eps_dum;
        sc_cost_with_eo([model_grad_cat_1; model_grad_cat_3; model_grad_cat_4], feature_grad_cat_2) = eps_dum;
        sc_cost_with_eo([model_grad_cat_1; model_grad_cat_2; model_grad_cat_4], feature_grad_cat_3) = eps_dum;
        sc_cost_with_eo([model_grad_cat_1; model_grad_cat_2; model_grad_cat_3], feature_grad_cat_4) = eps_dum;
        sc_cost = sc_cost_with_eo;        
    elseif (gradient_orient)
        model_grad_cat_1 = find(model_grads(:,1) > 0 & model_grads(:,2) >= 0);
        model_grad_cat_2 = find(model_grads(:,1) > 0 & model_grads(:,2) <= 0);
        model_grad_cat_3 = find(model_grads(:,1) < 0 & model_grads(:,2) >= 0);
        model_grad_cat_4 = find(model_grads(:,1) < 0 & model_grads(:,2) <= 0);

        feature_grad_cat_1 = find(feature_grads(:,1) > 0 & feature_grads(:,2) >= 0);
        feature_grad_cat_2 = find(feature_grads(:,1) > 0 & feature_grads(:,2) <= 0);
        feature_grad_cat_3 = find(feature_grads(:,1) < 0 & feature_grads(:,2) >= 0);
        feature_grad_cat_4 = find(feature_grads(:,1) < 0 & feature_grads(:,2) <= 0);

        sc_cost_with_eo = sc_cost;
        sc_cost_with_eo(model_grad_cat_1, [feature_grad_cat_2; feature_grad_cat_3; feature_grad_cat_4]) = eps_dum;
        sc_cost_with_eo(model_grad_cat_2, [feature_grad_cat_1; feature_grad_cat_3; feature_grad_cat_4]) = eps_dum;
        sc_cost_with_eo(model_grad_cat_3, [feature_grad_cat_1; feature_grad_cat_2; feature_grad_cat_4]) = eps_dum;
        sc_cost_with_eo(model_grad_cat_4, [feature_grad_cat_1; feature_grad_cat_2; feature_grad_cat_3]) = eps_dum;

        sc_cost_with_eo([model_grad_cat_2; model_grad_cat_3; model_grad_cat_4], feature_grad_cat_1) = eps_dum;
        sc_cost_with_eo([model_grad_cat_1; model_grad_cat_3; model_grad_cat_4], feature_grad_cat_2) = eps_dum;
        sc_cost_with_eo([model_grad_cat_1; model_grad_cat_2; model_grad_cat_4], feature_grad_cat_3) = eps_dum;
        sc_cost_with_eo([model_grad_cat_1; model_grad_cat_2; model_grad_cat_3], feature_grad_cat_4) = eps_dum;
        sc_cost = sc_cost_with_eo;
    else
        model_grad_cat_1 = find(abs(model_grads(:,1)) > abs(model_grads(:,2)));
        model_grad_cat_2 = find(abs(model_grads(:,1)) <= abs(model_grads(:,2)));
        feature_grad_cat_1 = find(abs(feature_grads(:,1)) > abs(feature_grads(:,2)));
        feature_grad_cat_2 = find(abs(feature_grads(:,1)) <= abs(feature_grads(:,2)));
        sc_cost_with_eo = sc_cost;
        sc_cost_with_eo(model_grad_cat_1, feature_grad_cat_2) = eps_dum;
        sc_cost_with_eo(model_grad_cat_2, feature_grad_cat_1) = eps_dum;
        sc_cost = sc_cost_with_eo;
    end
end
% compute min-cost matches   
model_to_features = viterbi_match(sc_cost, model_pts, feature_pts, continuity_coeff, curvature_coeff, dummy_pt);

end

