function [matching] = viterbi_match(cost_sc, model_pts, feature_pts, continuity_coeff, curvature_coeff, dummy_pt)
%Viterbi match perform the dynamic programming algorithm outlined in
%Thayananthan et al. CVPR03

%cost_sc is the cost matrix for the shape context matching
%cost_sc(i,j) == histogram cost of model pt (i) and feature pt (j)
%initialization:
c_all = zeros(size(model_pts,1),size(feature_pts,1), size(feature_pts, 1));
c_all_top = ones(size(cost_sc,1),size(cost_sc,2))*Inf;

n = size(model_pts,1);
m = size(feature_pts,1);
%This is the point that will be matched to if there is no good match in the
%Viterbi algorithm.
feature_pts_dist = sqrt(dist2(feature_pts,feature_pts));
mean_feature_dist = mean2(feature_pts_dist);
%Scale the feature points distance array by the mean distance, this has the
%effect of regularizing the continuity computation so that the continuity
%coefficient only has to vary as a model parameter.
feature_pts_dist = feature_pts_dist ./ mean_feature_dist;

model_pts_curvature = curv(model_pts);
feature_pts_curvature = curv(feature_pts);

%initialization
c_all_top(1,:) = cost_sc(1,:);
ptrs = zeros(max([size(model_pts,1), size(feature_pts,1)]));

%propagation:
for i=2:n
     for j=1:m
        %Hardcore matlab:
        c_all(i,j,:) = c_all_top(i-1,:)' + cost_sc(i,j) + continuity_coeff*(feature_pts_dist(:,j)) + curvature_coeff*(abs(feature_pts_curvature(:) - model_pts_curvature(i-1)));
        [c_all_top(i,j), ptrs(i,j)] = min(c_all(i,j,:));        
    end   
end

%termination:
[~, u_n] = min(c_all_top(n,1:m));
matching(n) = u_n;
u_ip1 = u_n;
for i=n-1:-1:1
    u_i = ptrs(i, u_ip1);
    if (sum(matching == u_i) > 0)
        matching(i) = dummy_pt;
    else
        matching(i) = u_i;
    end
    u_ip1 = u_i;
end


return;


function [curvature] = curv(pts)
%This function assumes that points are ordered adjacently

uim1 = [zeros(1,2); pts(1:end-1,:)];
twoui = 2.*pts(1:end,:);
uip1 = [pts(2:end,:); zeros(1,2)];
ku = uim1 - twoui + uip1;
curvature = [sqrt(sum(ku.*ku,2))];
