function [edgevote_map_volume_max, edgevote_map_volume_min]...
    = getRelevantContoursFromFixations(maxTorqueSegmentation_dat_i,minTorqueSegmentation_dat_i,...
        img_curve, objcat_dat, labeled_img_curve, objsize_vect, para)
% Derive an edge vote from torque fixations
%
% USAGE
%   [weighted_EM_Max, weighted_EM_Min, edgevote_map_volume_max, edgevote_map_volume_min, editDist_maps]...
%       = getRelevantContoursFromFixations(maxTorqueSegmentation_dat_i,minTorqueSegmentation_dat_i, img_curve, objcat_dat, labeled_img_curve, objsize_vect, para)
%
% INPUTS
%   maxTorqueSegmentation_dat_i     - maxima torque fixation data {Fx1}
%   minTorqueSegmentation_dat_i     - minima torque fixation data {Fx1}
%   img_curve                       - edge image [HxW]
%   objcat_dat                      - model data {4x1}
%   labeled_img_curve               - labeled contour image [HxW]
%   objsize_vect                    - expected object size [1x2]
%   para                            - parameters (see getPara.m)
%
% OUTPUTS
%   edgevote_map_volume_max         - edge votes from maxima torque [HxWxF]
%   edgevote_map_volume_min         - edge votes from minima torque [HxWxF]
%
% Copyright (C) 2014 Ching L Teo, University of Maryland College Park, [cteo-at-cs.umd.edu]
%
%    You can redistribute and/or modify this software for non-commercial use
%    under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%    For commercial use, contact the author for licensing options. 
%
% Please email me if you find bugs, or have suggestions or questions!
    
%%
[rIm, cIm] = size(img_curve);
nMinFixations = size(minTorqueSegmentation_dat_i, 1); nMaxFixations = size(maxTorqueSegmentation_dat_i, 1);
mTorqueSegmentation_dat_i = [minTorqueSegmentation_dat_i; maxTorqueSegmentation_dat_i];
nFixations = size(mTorqueSegmentation_dat_i, 1);
% save all location information to compute relative distance
locFixations = zeros(nFixations, 2);
for jj=1:nFixations,locFixations(jj,:) = mTorqueSegmentation_dat_i{jj}{1,5}; end
indLocFixations = sub2ind([rIm, cIm], locFixations(:,2), locFixations(:,1));
% store edgevotemaps together
% set default values
edgevote_map_volume_max = nan(rIm, cIm);edgevote_map_volume_min = nan(rIm, cIm);
max_labelval = max(labeled_img_curve(:));
if nFixations % if we do have any fixations at all 
    % precompute the edgevotes for each contour separately from
    % labeled_img_curve
    nContours = max(labeled_img_curve(:)); edgevotes_indv = nan(rIm, cIm, nContours);
    edgevote_map_volume = nan(rIm, cIm, nFixations);    
    % Use objsize vect as to set the mask size to consider
    max_width = objsize_vect(1,1); max_height = objsize_vect(1,2);
    %%
    parfor i=1:nFixations
        segdat_curr = mTorqueSegmentation_dat_i{i,1}; torqueCenter_c = segdat_curr{1,5}; %[xcenter,ycenter]
        % compute region of interest (based on approximate target BB)
        segdat_loc = segdat_curr{1,5}; jmax = segdat_loc(1); imax = segdat_loc(2);
        bb_max_tl_x = ceil(max(1,jmax-max_width/2)); % topleft_x
        bb_max_tl_y = ceil(max(1,imax-max_height/2)); % topleft_y
        bb_max_br_x = floor(min(cIm, jmax+max_width/2)); %bottomright_x
        bb_max_br_y = floor(min(rIm, imax+max_height/2)); % bottomright_y        
        im_mask = false(rIm, cIm);
        im_mask(bb_max_tl_y:bb_max_br_y, bb_max_tl_x:bb_max_br_x) = true;    
        % find all fixations within im_mask
        ind_tt = find(im_mask > 0); tf_in = ismember(indLocFixations, ind_tt, 'rows');        
        fixPts_in = find(tf_in == 1); nFixPts_in = size(fixPts_in, 1);
        % combine all contours from fixPts_in
        segpos_comb = false(rIm, cIm); segcont_comb = false(rIm, cIm);
        edgevotes_comb = nan(rIm, cIm, (nFixPts_in*2+1)); edgevotes_indv_n = [];   % temp results of all edgevotes   
        ff=1;
        %%
        for kk=1:nFixPts_in
            ind_c = fixPts_in(kk);segdat_cc = mTorqueSegmentation_dat_i{ind_c,1};
            % select strongest torque contributions
            edgecont_bw = segdat_cc{1,1} > para.sc.segdat_factor*nanmax(segdat_cc{1,1}(:));     
            labels_edgecont = labeled_img_curve.*edgecont_bw; labels_hist = histc(labels_edgecont(:), 1:max_labelval);
            labels_ind_nz = find(labels_hist > para.sc.minlength);
            seg_cont = false(rIm, cIm);nlabels_nz = size(labels_ind_nz, 1);
            for ii=1:nlabels_nz
                cc_ind = labels_ind_nz(ii);cc_count = labels_hist(cc_ind);
                if cc_count > para.sc.minlength % significant counts of this contour.
                    ind_cc = labeled_img_curve == cc_ind;seg_cont(ind_cc) = true;
                    edgevotes_indv_n = [edgevotes_indv_n; cc_ind];  % save valid contour indices                                   
                end
            end
            if sum(seg_cont(:)) > 0 && ind_c == i % current fixation
                rotAngle_V = estimateEdgeRotate_FFT(seg_cont, objcat_dat); % FFT radon angle determination 
                edgevotes_comb(:,:,ff) = computeEdgeVotesFromSegment_FFT(seg_cont,objcat_dat, torqueCenter_c, rotAngle_V, para);                 
            end
            ff=ff+1;
            % combine together 
            segcont_comb = or(segcont_comb, seg_cont);segpos_comb = or(segpos_comb, segdat_cc{1,2});
            ff=ff+1;            
        end  % end kk=1:nFixPts_in 
        %%
        % compute edgevotes from segments (all combined)       
        if sum(segcont_comb(:)) > 0
            rotAngle_V = estimateEdgeRotate_FFT(segcont_comb, objcat_dat);% estimate rotation
            % run edgevotes per fix pts
            edgevotes_comb(:,:,ff) = computeEdgeVotesFromSegment_FFT(segcont_comb,objcat_dat, torqueCenter_c, rotAngle_V, para);                 
            % do the same for individual contours -- saved in edgevotes_indv_n
            edgevotes_indv_nu = unique(edgevotes_indv_n); nContours_indv = size(edgevotes_indv_nu,1);
            edgevotes_indv_comb_c = nan(rIm, cIm, nContours_indv);
            for qq=1:nContours_indv,edgevotes_indv_comb_c(:,:,qq) = edgevotes_indv(:,:,edgevotes_indv_nu(qq));end
            edgevotes_comb_all = cat(3, edgevotes_comb, edgevotes_indv_comb_c);
            edgevote_map_volume(:,:,i) = nanmax(edgevotes_comb_all, [], 3);% get the top votes from all the permutations        
        else
            edgevote_map_volume(:,:,i) = nan(rIm, cIm);
        end
    end % end i=1:nFixations  
    edgevote_map_volume_min = edgevote_map_volume(:,:,1:nMinFixations);edgevote_map_volume_max = edgevote_map_volume(:,:,nMinFixations+1:end);  
end

end