function [edgePts_s_c] = centerEdgesToModel(edge_Im, model_curves_cell, para)
% center an input edge map from the training sequence to the model, using
% procrustes
%
% USAGE
%   [edgePts_s_c] = centerEdgesToModel(edge_Im, model_curves_cell, para)
% 
% INPUTS
%   edge_Im                 - model logical edge image [IxJ]
%   model_curves_cell       - coordinates [y,x] of extracted C contours {Cx1}
%   para                    - parameters (see getPara.m)
% 
% OUTPUTS
%   edgePts_s_c             -  centered edge points to model [Kx2]
%
% Copyright (C) 2014 Ching L Teo, University of Maryland College Park, [cteo-at-cs.umd.edu]
%
%    You can redistribute and/or modify this software for non-commercial use
%    under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%    For commercial use, contact the author for licensing options. 
%
% Please email me if you find bugs, or have suggestions or questions!

% get curves from edges
curve = extract_curve(edge_Im,para.edge.gapsize, para.edgeLength);  
% sanity check -- must have one curve
if size(curve, 1) > 1,error('input edge image has more than 1 contour');end
% now compare the curve with the model, taking the one with the smallest
% procrustes distance
Dist_v_mat = zeros(size(model_curves_cell,1), 1);
Z_cell = cell(size(model_curves_cell,1), 1);
for kk=1:size(model_curves_cell,1)
    cc_t = model_curves_cell{kk,1};
    curve_s_t = smoothContour_edges(curve{1}, para, size(cc_t,1));
    % do matlab procrustes since the distance measure is better
    [Dist_v_mat(kk,1),~,~] = procrustes(cc_t, curve_s_t, 'scaling', true, 'reflection', 'best');
    % use the OPA transform
    [Z_cell{kk,1},~,~] = OPA(cc_t,curve_s_t, true);    
end

% find minimum distance model
[~, iMin] = min(Dist_v_mat);
edgePts_s_c = Z_cell{iMin,1}; % [y, x]

end