function [curve_new, corner_coord, indx_corner] = computeLocalCurvature(curve_curr, para)
% computes local curvature of the input curve using splines as smoothing
% input, output new curve, corner_coordinates, and indices of corners in
% new curve
%
% USAGE
%   [curve_new, corner_coord, indx_corner] = computeLocalCurvature(curve_curr, para)
% 
% INPUTS
%   curve_curr              - smoothed edge pts [Kx2]
%   para                    - parameters (see getPara.m)
% 
% OUTPUTS
%   curve_new               - smoothed curved [Lx2]
%   corner_coord            - coordinates [y,x] of large curvatures [CCx2]
%   indx_corner             - indices of large curvatures [CCx1]
%
% Copyright (C) 2014 Ching L Teo, University of Maryland College Park, [cteo-at-cs.umd.edu]
%
%    You can redistribute and/or modify this software for non-commercial use
%    under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%    For commercial use, contact the author for licensing options. 
%
% Please email me if you find bugs, or have suggestions or questions!

% smooth contours via splines, and increase num of points to twice the
% number of curves points
curve_s = smoothContour_edges(curve_curr, para, size(curve_curr,1)*para.edge.interpMult);
curve_s = [curve_s(:,2), curve_s(:,1)]; % [x, y] again
% 2D spline
knots = [curve_s(:,1)'; curve_s(:,2)'];    
% create spline in pp form
orig_space = 1:size(curve_s,1); pp = spline(orig_space, knots);
% % first derivative, pp form
pp1 = pp;pp1.order = pp.order-1; 
pp1.coefs = bsxfun(@times, pp.coefs(:,1:end-1), pp1.order:-1:1);
% second derivative, pp form
pp2 = pp1;pp2.order = pp1.order-1; 
pp2.coefs = bsxfun(@times, pp1.coefs(:,1:end-1), pp2.order:-1:1);
% compute derivatives
xx = orig_space;A1 = ppval(pp1, xx);A2 = ppval(pp2, xx);
% compute local curvature
curvature = (A1(1,:).*A2(2,:) - A1(2,:).*A2(1,:))./(A1(1,:).^2 + A1(2,:).^2).^(3/2);
% get local max and min
[cmax,ind_cmax,cmin,ind_cmin] = extrema(curvature);
% ilter values
thres_val = std(curvature)*0.5;
f_indmin = ind_cmin(abs(cmin-mean(curvature)) > thres_val);
f_indmax = ind_cmax(abs(cmax-mean(curvature)) > thres_val);
% combine max and min indices
f_indcomb = [f_indmin, f_indmax];f_indcomb_s = sort(f_indcomb);
f_indcomb_s = [1, f_indcomb_s, size(curve_s,1)]; % add endpoints indices

% remove fragments that have angles that are too near to 180 deg
% (estimate from +/- 10 pixels apart)
rmv_inda = [];keep_inda = [];
for i=2:size(f_indcomb_s,2)-1 % exclude endpoints
    c_pt = curve_s(f_indcomb_s(i),:);s_pt = curve_s(max(1,f_indcomb_s(i)-10),:);
    e_pt = curve_s(min(size(curve_s,1),f_indcomb_s(i)+10),:);    
    vcs = s_pt - c_pt;vce = e_pt - c_pt; 
    angle = acos(dot(vcs,vce)./(norm(vcs).*norm(vce)))*(180/pi);
    if angle > 175
        rmv_inda = [rmv_inda, i];
    else
        keep_inda = [keep_inda, i];
    end
    
end
f_indcomb_af = [1, f_indcomb_s(keep_inda), size(curve_s,1)]; % add endpts
length_seg = diff(f_indcomb_af);
rmv_ind = [];keep_ind = [];
for i=1:length(length_seg)-1 % no need to check last segment
    length_curr = length_seg(i)/para.edge.interpMult;
    thresh_length = (0.1*size(curve_s,1))/para.edge.interpMult;
    if length_curr < thresh_length % if too short, remove
        rmv_ind = [rmv_ind, i+1];
        length_seg(i+1) = length_seg(i) + length_seg(i+1); % update
    else
        keep_ind = [keep_ind, i+1];
    end
end
f_indcomb_final = f_indcomb_af(keep_ind);

% return curves, corner_coord, indx_corner
curve_new = fliplr(curve_s); % [y,x]
corner_coord = fliplr(curve_s(f_indcomb_final,:));indx_corner = f_indcomb_final';

end