% Script to extract model contours given a set (here 1) training shape
% masks. Cluster contours via B-splines.
% NOTE: run this to save 'train_ObjClassContours_MODEL_NAME.mat' needed in
% demo.m
%
% If you use this code for academic purposes, please cite:
% Ching L. Teo, C. Fermüller and Y. Aloimonos. 
% "A Gestaltist approach to contour-based object recognition: Combining bottom-up and top-down cues". 
% International Journal of Robotics Research (IJRR), Accepted (2014)
%
% Copyright (C) 2014 Ching L Teo, University of Maryland College Park, [cteo-at-cs.umd.edu]
%
%    You can redistribute and/or modify this software for non-commercial use
%    under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%    For commercial use, contact the author for licensing options. 
%
% Please email me if you find bugs, or have suggestions or questions!

clear all;
close all;

%% setup
addpath(genpath(fullfile('../'))); addpath(genpath(fullfile('../external/')));
para = getPara(); para.test.dir_suffix = 'Saw2';
% model handdrawn mask
model_hd =  im2bw(imread(fullfile('../data/models', [para.test.dir_suffix '.jpg']))); model_hd = ~model_hd; 
% preprocess the model_hd to a model_curve (for centering)
[model_curves_cell] = preprocessModel(model_hd, para);
nProc = 1; % simplified case -- we only have one training data
% structure to save the masks and current contours
msk_imgs_c = cell(nProc, 1); objcat_salcont_c = cell(nProc, 1); % saves current extract salient contour pieces per mask
% get number of regions 
im_c=model_hd; SS=regionprops(im_c, 'PixelList'); nRegions=size(SS,1);
%%
parfor i=1:nProc
    contour_frag = [];
    for kk=1:nRegions
        im_t = false(size(im_c));       
        indx_t = sub2ind(size(im_t), SS(kk,1).PixelList(:,2), SS(kk,1).PixelList(:,1));
        im_t(indx_t) = true;edge_Im = bwperim(im_t,4);
        % create centered edges to model
        [edge_pts_c] = centerEdgesToModel(edge_Im, model_curves_cell, para);
        % get salient contours
        cf_tmp = getSalientFragments(edge_pts_c, para);
        for ll=1:size(cf_tmp,1),contour_frag{end+1,1} = cf_tmp{ll,1};end
    end
    contour_frag_s = cell(size(contour_frag,1),1);
    % makes sure each contour_frag is of the standard length
    for jj=1:size(contour_frag,1)
        cont_c = contour_frag{jj,1};
        if size(cont_c,1) > para.edge.contourMinLength %set to 1 since we do not filter this here anymore
            cont_s = smoothContour_edges(cont_c, para);contour_frag_s{jj,1} = cont_s;
        end
    end
    objcat_salcont_c{i,1} = contour_frag_s;
    msk_imgs_c{i,1} = im_c; % saves mask_imgs
end % end i=1:nProc
%% construct a cell matrix containing all the non-empty splines
Data_cell = {};avg_cont = zeros(size(objcat_salcont_c,1),1);
for i=1:size(objcat_salcont_c,1)
    curr_cell = objcat_salcont_c{i,1};avg_cont(i,1) = sum(~cellfun('isempty',curr_cell));
    for j=1:size(curr_cell,1)
        if ~isempty(curr_cell{j,1}),Data_cell{end+1} = curr_cell{j,1};end
    end 
end
Data_cell = Data_cell';
% get averaged contours -- set number of clusters needed
nMean_cont = ceil(mean(avg_cont));
% Run EM clustering of data -- get mm (estimated model clusters)
ops = lrm('options'); ops.method = para.EM.ops.method; ops.K = nMean_cont;
ops.NumEMStarts = para.EM.NumEMStarts;ops.IterLimit = para.EM.ops.IterLimit;
ops.order = para.EM.ops.order;ops.MsgHnd = -1;
mm = curve_clust(Data_cell, ops);
%show cluster results
showmodel(mm, Data_cell);
Data_clusterRes = mm.C;nClusters = mm.K;rCoeffs = mm.Mu;
%% gather all splines within one cluster and get "averaged spline" using GPA
for i=1:nClusters
    ind_c = Data_clusterRes == i;cont_clust = Data_cell(ind_c);
    if size(cont_clust, 1) > 1
        % put into a 2x para.edgefeatures.splinelength x size(cont_clust,1)
        % matrix
        data_matGPA = zeros(2, para.edgefeatures.splinelength, size(cont_clust,1));
        for j=1:size(cont_clust,1)
           data_matGPA(1,:,j) = cont_clust{j,1}(:,2)'; %[x]
           data_matGPA(2,:,j) = cont_clust{j,1}(:,1)'; %[y]          
        end
        % perform GPA
        [avgSpline_GPA, Gval] = GPA(data_matGPA, true);
        avgSpline_res = mean(avgSpline_GPA,3);
    elseif size(cont_clust, 1) == 1
        avgSpline_res = [cont_clust{1,1}(:,2)'; cont_clust{1,1}(:,1)'] ;
    end
    if size(cont_clust, 1) > 0
        % save into cell for each cluster
        avgSpline_cell_c{i,1} = [avgSpline_res(2,:)', avgSpline_res(1,:)']; % [y x]
    end

end % end i=1:nClusters
%%
slength_v = zeros(size(avgSpline_cell_c,1),1);
for ii=1:size(avgSpline_cell_c,1)
    s_ii = avgSpline_cell_c{ii,1};l_ii = computeSplineLength(s_ii);
    slength_v(ii,1) = l_ii;
end
% remove splines that are too short
ii_vv = slength_v > para.train.minsplinelength;
avgSpline_cell_cv = avgSpline_cell_c(ii_vv);
% save mask images and avgSplines per instance into cell
msk_img_cell{1,1} =  msk_imgs_c;objcat_contours{1,1} = avgSpline_cell_cv;
% remove contours that are repeated
nCont = size(objcat_contours{1,1},1); ind_rmv = [];
for kkk = 1:nCont
    cont_c = objcat_contours{1,1}{kkk,1};
    for jjj=1:nCont
        if jjj==kkk, continue;
        else
            cont_cc = objcat_contours{1,1}{jjj,1};
            if sum(abs(cont_c - cont_cc)) <= 1e-3,ind_rmv = [ind_rmv; kkk, jjj];end
        end
    end         
end
if ~isempty(ind_rmv)
    nEle = size(ind_rmv,1);ind_rmv(1:nEle/2,:) = fliplr(ind_rmv(1:nEle/2,:));
    ind_rmv_u = unique(ind_rmv, 'rows');ind_rmv_f = ind_rmv_u(:,1);
    objcat_contours_n = cell(nCont-size(ind_rmv,1),1);
    ll=1;
    for ppp=1:nCont
        if ~ismember(ppp,ind_rmv_f)
            objcat_contours_n{ll,1} = objcat_contours{1,1}{ppp,1};
            ll=ll+1;
        end
    end
    objcat_contours{1,1} = {};objcat_contours{1,1} = objcat_contours_n;
end

% combine all contours into one large cell_mat
mat_comb = squeezeCells(objcat_contours); figure, hold on; cmapL=colormap(lines);
for i=1:size(mat_comb, 1)
    plot(mat_comb{i}(:,2), mat_comb{i}(:,1), 'Color', cmapL(i,:), 'LineStyle', 'none', 'Marker', '*');
end
title(sprintf('number of contours: %d', size(mat_comb,1))); set(gca,'Ydir','reverse');

%% save the contours and mask images
save(['train_ObjClassContours_' para.test.dir_suffix '.mat'], 'objcat_contours', 'msk_img_cell');    
